<?php
// PI_Library.php
// based on http://www.netzmafia.de/skripten/hardware/RasPi/RasPi_GPIO_C.html
// by Prof. Juergen Plate
// ACHTUNG einige Funktionen benötigen root Rechte
// ergaenzt von mk
// optionale Funktionen einbauen
// active_low_high, edge, power, uevent

require_once('ALL_Constants.php');

/**
 * exportiert GPIO 
 *
 * @param GPIO number $pin
 * @return string
 */
function gpio_enable($pin)
// inititialisiert (enabled) GPIO
// extra by mk
{
	$erg = gpio_is_exported($pin);
	if ($erg)		// Test if already enabled
	{
		$Dir = gpio_get_direction($pin);
	}
	else
	{
		$filename = GPIOPATH.'export';
		file_put_contents($filename, $pin);
		$Dir ='';
	}
	return($Dir);
}


/**
 * löscht einen GPIO
 *
 * @param GPIO Nummer $pin
 */
function gpio_disable($pin)
{
	if (gpio_is_exported($pin))		// Test if already enabled
	{
		gpio_set_value($pin,'0');
		
		file_put_contents(GPIOPATH.'unexport', $pin);
	}
	return;
}


/**
 * prueft ob Pin exportiert ist
 * 
 * @param GPIO Nummer $pin
 * @return boolean
 */
function gpio_is_exported($pin)
{
	$path = GPIOPATH.'gpio'.$pin.'';
	if (file_exists($path))
	{
		return true;
	}
	else 
	{
		return false;
	}
}
	
/**
 * gibt die eingestellte Richtung zurueck
 *
 * @param string $pin
 * @return string richtung
 */
function gpio_get_direction($pin)
{
	$dir = '';
	if (gpio_is_exported($pin))
	{
		$filename = GPIOPATH.'gpio'.$pin.'/direction';
		$dir = trim(file_get_contents($filename));
	}
	return $dir;
}
  
/**
 * Stellet GPIO auf ein- bzw. Ausgang
 *
 * @param GPIO Number $pin
 * @param string $direction
 * @return boolean
 */
function gpio_set_direction($pin, $direction)
// Stellt die Richtung ein ($direction: 'in' oder 'out')
{
	if (gpio_is_exported($pin) && ($direction == 'in' || $direction == 'out'))
	{
		file_put_contents(GPIOPATH.'gpio'.$pin.'/direction', $direction);
		return true;
	}
	else
	{
		return false;
	}
}

/**
 * gibt den augenblicklichen Wert (0/1) des Pins zurueck
 * oder einen leeren String im Fehlerfall
 *
 * @param GPIO Nummer $pin
 * @return string richtung
 */
function gpio_get_value($pin)
{
	$Val = '';
	if (gpio_is_exported($pin));// && gpio_get_direction($pin) == 'in')
	{
		$Val = trim(file_get_contents(GPIOPATH.'gpio'.$pin.'/value'));
	}
	return $Val;
}

/**
 * Setzt Zustand des GPIO  1 oder 0
 *
 * @param GPIO number $pin
 * @param Wert $value
 * @return Boolean
 */
function gpio_set_value($pin, $value)
{
	if (gpio_is_exported($pin) && gpio_get_direction($pin) == 'out' && ($value == '0' || $value == '1'))
	{
		$filename = GPIOPATH.'gpio'.$pin.'/value';
		$erg = file_put_contents($filename, $value);
		return true;
	}
	else
	{
		return false;
	}
}

/**
 * zum testen, alternative in PHP als Ersatz für gpio readall aus dem wiringPi Paket
 * Schmalspurversion, kann ausgebaut werden
 * wiringPi muss nicht installiert werden
 * optionaler Parameter gibt an statt CRLF '<br> ausgegeb werden soll
 *
 * @param boolean $br
 */
function gpio_readall($br=false)
// zum testen, alternative in PHP als Ersatz für gpio readall aus dem wiringPi Paket
// 
// 

{
	if (!$br)
	{
		$cr = chr(0x0D).chr(0x0A);
	}
	else 
	{
		$cr ='<br>';
	}
	echo 'List GPIO similar to CMDLine "gpio readall" of wireingPi'.$cr;
	echo 'GPIO_x = wPI, BCM x = BCM Value, Pin x = physical pin header'.$cr;
	$ports = array(GPIO_0,GPIO_1,GPIO_2,GPIO_3,GPIO_4,GPIO_5,GPIO_6,GPIO_7);
	$pins = array(11,12,13,15,16,18,22,7);
	$offset = 0;		//offset GPIO Numbers
	gpio_readall_sub($ports,$pins,$offset,$br);
	$ports = array(GPIO_21,GPIO_22,GPIO_23,GPIO_24,GPIO_25,GPIO_26,GPIO_27,GPIO_28,GPIO_29);
	$pins = array(29,31,33,35,37,32,36,38,40);
	$offset = 21;		//offset GPIO Numbers
	gpio_readall_sub($ports,$pins,$offset,$br);
}
	
/**
 * Hilfsprogramm zu gpio_readall
 * macht die eigentliche Ausgabe 
 *
 * @param array $ports
 * @param array $pins
 * @param int $offset
 * @param boolean $br
 */
function gpio_readall_sub($ports,$pins,$offset,$br)
{
	if (!$br)
	{
		$cr = chr(0x0D).chr(0x0A);
	}
	else 
	{
		$cr ='<br>';
	}
	$size = count($ports);
	for ($i=0;$i<$size;$i++)
	{
		echo 'GPIO_'.($i+$offset).' (BCM '.$ports[$i].') Pin '.$pins[$i].' ';						// offset GPIO numbers
		if (gpio_is_exported($ports[$i]))
		{
			if (gpio_get_direction($ports[$i])== 'out')
			{
				echo 'Direction OUT ';
			}
			else
			{
				echo 'Direction IN ';
			}
			echo 'Value '.gpio_get_value($ports[$i]);					
		}
		else 
		{
			echo 'NOT exported';
		}
		echo $cr;
	}
}

/**
 * Stellt Zustand über Interrupteinstellung fest
 * liefert leer im Fehlerfall, ansonsten none, rising,faling, both zurück
 *
 * @param unknown_type $pin
 * @return unknown
 */
function gpio_get_interrupt($pin)
{
	$function = '';
	if(gpio_is_exported($pin) && gpio_get_direction($pin) == 'in')
	{
		$filename = GPIOPATH.'gpio'.$pin.'/edge';
		$function = trim(file_get_contents($filename));
	}
	return $function;
}

/**
 * Enter description here...
 *
 * @param GPIO number $pin
 * @param string $function
 * @return boolean
 */
function gpio_set_interrupt($pin,$function)
{
	if(gpio_is_exported($pin) && gpio_get_direction($pin) == 'in' && ($function == 'none' || $function == 'rising' || $function == 'faling' || $function == 'both'))
	{
		$filename = GPIOPATH.'gpio'.$pin.'/edge';
		file_put_contents($filename, $function);
		return true;
	}
	else 
	{
		return false;
	}
}
   
?>
